/**
 * 处理对称计算类请求
 */


package com.sunyard.chsm.controller;
import cn.hutool.core.codec.Base64;
import com.sunyard.chsm.enums.AlgMode;
import com.sunyard.chsm.enums.KeyAlg;
import com.sunyard.chsm.enums.Padding;
import com.sunyard.chsm.service.KeyInfoService;
import com.sunyard.chsm.service.SYMEncryptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import params.DTORes;
import params.DTOSymReq;

import java.util.HashSet;
import java.util.Set;

@RestController
@RequestMapping("/sym")
public class SYMEncryptController {

    @Autowired
    private KeyInfoService keyInfoService;

    @Autowired
    private SYMEncryptService symEncryptService;

    // 支持的算法
    private static final Set<String> SUPPORTED_ALG = new HashSet<>();

    // 支持的模式
    private static final Set<String> SUPPORTED_MODE = new HashSet<>();

    // 支持的填充方式
    private static final Set<String> SUPPORTED_PADDING = new HashSet<>();

    static {
        SUPPORTED_ALG.add("SM4");
    }


    @PostMapping("/encrypt")
    @ResponseBody
    public DTORes encrypt(@RequestBody DTOSymReq req) {
        return doEnOrDe(true, req);
    }

    @PostMapping("/decrypt")
    @ResponseBody
    public DTORes decrypt(@RequestBody DTOSymReq req) {
        return doEnOrDe(false, req);
    }

    /**
     * 合并处理加密和解密逻辑
     * @param en true 为加密 false 为解密
     * @param req 请求数据
     * @return 返回数据
     */
    private DTORes doEnOrDe(boolean en, DTOSymReq req){
        // 参数检查
        // 填充默认参数
        if ( null == req.getAppId() ) {
            throw new IllegalArgumentException("appId can't be null");
        }

        // 算法检查
        if ( ! SUPPORTED_ALG.contains(req.getAlg())  ) {
            throw new IllegalArgumentException("alg must be SM4");
        }

        // 模式检查
        if ( ! SUPPORTED_MODE.contains(req.getMode()) ) {
            throw new IllegalArgumentException("mode must be ECB/CBC");
        }

        // 填充检查
        if ( ! SUPPORTED_PADDING.contains( req.getPadding() ) ) {
            throw new IllegalArgumentException("padding must be none or pkcs5");
        }


        // iv
        byte[] iv = null;
        if ( ! "ECB".equals(req.getMode()) ) {
            if ( null == req.getIv() ) {
                throw new IllegalArgumentException("iv can't be null");
            }
            iv = Base64.decode( req.getIv() );
        }

        // data
        if ( null == req.getData() ) {
            throw new IllegalArgumentException("data can't be null");
        }
        byte[] data = Base64.decode( req.getData() );

        // 功能执行
        if ( en ) {
            data = symEncryptService.symEncrypt(
                    KeyAlg.of( req.getAlg() ),
                    AlgMode.of( req.getMode() ),
                    Padding.of( req.getPadding() ),
                    req.getAppId(),
                    req.getKeyId(),
                    iv,
                    data
            );
        } else {
            data = symEncryptService.symEncrypt(
                    KeyAlg.of( req.getAlg() ),
                    AlgMode.of( req.getMode() ),
                    Padding.of( req.getPadding() ),
                    req.getAppId(),
                    req.getKeyId(),
                    iv,
                    data
            );
        }

        return DTORes.success(data);
    }
}
